############################################################
#
# <p>Gracenote</p>
# <pre>
# {{gracenote Perfume,GAME}}
# </pre>
#
############################################################
package plugin::gracenote::Gracenote;
use strict;
use feature ':5.10';
use LWP::UserAgent;
use XML::Simple;
use HTTP::Request;
use Encode qw/from_to/;
use Unicode::RecursiveDowngrade;
use Unicode::Japanese;

#===========================================================
# 󥹥ȥ饯
#===========================================================
sub new {
  my $class = shift;
  my $self = {};
  $self->{count} = 1;
  return bless $self,$class;
}

#===========================================================
# ѥ饰ե᥽å
#===========================================================
sub paragraph {
  my $self = shift;
  my $wiki = shift;
  my $artist = shift;
  my $album = shift;

  my $client_id = '0000000-00000000000000000000000000000000';
  my $user_id = '000000000000000000-00000000000000000000000000000000';

  my $url_head = $client_id;
  $url_head =~ s/-.*$//;

  my $message = <<"XML";
<QUERIES>
  <AUTH>
    <CLIENT>$client_id</CLIENT>
    <USER>$user_id</USER>
  </AUTH>
  <LANG>jpn</LANG>
  <QUERY CMD="ALBUM_SEARCH">
    <MODE>SINGLE_BEST_COVER</MODE>
    <TEXT TYPE="ARTIST">$artist</TEXT>
    <TEXT TYPE="ALBUM_TITLE">$album</TEXT>
    <OPTION>
      <PARAMETER>SELECT_EXTENDED</PARAMETER>
      <VALUE>COVER,ARTIST_BIOGRAPHY,ARTIST_IMAGE,REVIEW</VALUE>
      <PARAMETER>COVER_SIZE</PARAMETER>
      <VALUE>SMALL,MEDIUM,THUMBNAIL</VALUE>
    </OPTION>
  </QUERY>
</QUERIES>
XML

  from_to($message,"euc-jp","utf8");

  my $ua = LWP::UserAgent->new;
  my $url = "https://c".$url_head.".web.cddbp.net/webapi/xml/1.0/";
  my $req= HTTP::Request->new(POST => $url);
  $req->content($message);
  my $response = $ua->request($req);

  if ($response->is_error){
    return $response->content;
  }

  my $rd = Unicode::RecursiveDowngrade->new;
  $rd->filter(sub {Unicode::Japanese->new(shift,'utf8')->euc});
  my $data = $rd->downgrade(XMLin($response->content));

  my $out = "<div class=\"gracenote\"><table>";

  my $coverart = "";
  my $artist_biography = "";
  my $artist_image = "";
  my $review = "";

  my $track_count = $data->{RESPONSE}{ALBUM}{TRACK_COUNT};
  my $table_lines = $track_count + 3;

  if(ref $data->{RESPONSE}{ALBUM}{URL} ne 'ARRAY') {
    given ($data->{RESPONSE}{ALBUM}{URL}{TYPE}) {
      my $val = $data->{RESPONSE}{ALBUM}{URL}{content};
      when ('COVERART') {
        $coverart = $val;
      }
      when ('ARTIST_BIOGRAPHY') {
        $artist_biography = $val;
      }
      when ('ARTIST_IMAGE') {
        $artist_image = $val;
      }
      when ('REVIEW') {
        $review = $val;
      }
    }
  } else {
    for my $num (0..3) {
      given ($data->{RESPONSE}{ALBUM}{URL}[$num]{TYPE}) {
        my $val = $data->{RESPONSE}{ALBUM}{URL}[$num]{content};
        when ('COVERART') {
          $coverart = $val;
        }
        when ('ARTIST_BIOGRAPHY') {
          $artist_biography = $val;
        }
        when ('ARTIST_IMAGE') {
          $artist_image = $val;
        }
        when ('REVIEW') {
          $review = $val;
        }
      }
    }
  }

  if ($artist_image) {
    $out .= "<tr><td valign=\"top\"><a href=\"$artist_image\" target=\"_blank\"><img width=\"160\" src=\"$artist_image\" alt=\"$data->{RESPONSE}{ALBUM}{ARTIST}\"></a></td>";
  } else {
    $out .= "<tr><td valign=\"top\"></td>";
  }
  if ($artist_biography) {
    $out .= "<td valign=\"top\">Artist:</td><td valign=\"top\"><a href=\"$artist_biography\" target=\"_blank\">$data->{RESPONSE}{ALBUM}{ARTIST}</a></td></tr>";
  } else {
    $out .= "<td valign=\"top\">Artist:</td><td valign=\"top\">$data->{RESPONSE}{ALBUM}{ARTIST}</td></tr>";
  }
  if ($coverart) {
    $out .= "<tr><td valign=\"top\" rowspan=\"$table_lines\"><a href=\"$coverart\" target=\"_blank\"><img width=\"160\" src=\"$coverart\" alt=\"$data->{RESPONSE}{ALBUM}{TITLE}\"></a></td>";
  } else {
    $out .= "<tr><td valign=\"top\" rowspan=\"$table_lines\"></td>";
  }
  if ($review) {
    $out .= "<td>Album:</td><td valign=\"top\"><a href=\"$review\" target=\"_blank\">$data->{RESPONSE}{ALBUM}{TITLE}</a></td></tr>";
  } else {
    $out .= "<td>Album:</td><td valign=\"top\">$data->{RESPONSE}{ALBUM}{TITLE}</td></tr>";
  }
  $out .= "<td>Date:</td><td>$data->{RESPONSE}{ALBUM}{DATE}</td></tr>";

  my $head = "Track:";
  for my $num (0..$track_count){
    $out .= "<td>$head</td><td>$data->{RESPONSE}{ALBUM}{TRACK}[$num]{TITLE}</td></tr>";
    $head = "";
  }

  $out .= "</table></div>";

  return $out;
}

1;

